@@ -32,10 +32,13 @@ common_format_config = function(
   # provide file_scope if requested
   if (file_scope) config$file_scope = md_chapter_splitter
 
-  # prepend the custom-environment filter
-  config$pandoc$lua_filters = c(
-    lua_filter("custom-environment.lua"), config$pandoc$lua_filters
-  )
+  # prepend the custom-environment filter unless opt-out
+  if (getOption("bookdown.theorem.enabled", TRUE)) {
+    config$pandoc$lua_filters = c(
+      lua_filter("custom-environment.lua"),
+      config$pandoc$lua_filters
+    )
+  }
   # and add bookdown metadata file for the filter to work
   config$pandoc$args = c(bookdown_yml_arg(), config$pandoc$args)
 
@@ -47,6 +50,12 @@ common_format_config = function(
   # when the output is LaTeX, force LaTeX tables instead of default Pandoc tables
   # http://tex.stackexchange.com/q/276699/9128
   config$knitr$opts_knit$kable.force.latex = TRUE
+
+  # deactivate header attributes handling from rmarkdown
+  # as done in bookdown::clean_html_tag()
+  opts <- options(rmarkdown.html_dependency.header_attr = FALSE)
+  config$on_exit <- function() options(opts)
+
   config
 }
 
@@ -60,10 +69,11 @@ get_base_format = function(format, options = list()) {
   do.call(format, options)
 }
 
-load_config = function() {
-  if (length(opts$get('config')) == 0 && file.exists('_bookdown.yml')) {
+load_config = function(config_file = '_bookdown.yml') {
+  config_file = opts$get('config_file') %n% config_file
+  if (length(opts$get('config')) == 0 && file.exists(config_file)) {
     # store the book config
-    opts$set(config = rmarkdown:::yaml_load_file('_bookdown.yml'))
+    opts$set(config = rmarkdown:::yaml_load_file(config_file))
   }
   opts$get('config')
 }
@@ -120,8 +130,6 @@ output_dirname = function(dir, config = load_config(), create = TRUE) {
   dir
 }
 
-dir_exists = function(x) utils::file_test('-d', x)
-
 # mark directories with trailing slashes
 mark_dirs = function(x) {
   i = dir_exists(x)
@@ -129,13 +137,6 @@ mark_dirs = function(x) {
   x
 }
 
-# TODO: use xfun::del_empty_dir()
-clean_empty_dir = function(dir) {
-  if (is.null(dir) || !dir_exists(dir)) return()
-  files = list.files(dir, all.files = TRUE, recursive = TRUE)
-  if (length(files) == 0) unlink(dir, recursive = TRUE)
-}
-
 merge_chapters = function(files, to, before = NULL, after = NULL, orig = files) {
   # in the preview mode, only use some placeholder text instead of the full Rmd
   preview = opts$get('preview'); input = opts$get('input_rmd')
@@ -262,16 +263,13 @@ clean_meta = function(meta_file, files) {
 
 # remove HTML tags and remove extra spaces
 strip_html = function(x) {
-  x = gsub('<!--.*?-->', '', x)  # remove comments
-  x = gsub('<[^>]+>', '', x)
-  x = gsub('\\s{2,}', ' ', x)
-  x
+  gsub('\\s{2,}', ' ', xfun::strip_html(x))
 }
 
 # remove the <script><script> content and references
 strip_search_text = function(x) {
   x = gsub('<script[^>]*>(.*?)</script>', '', x)
-  x = gsub('<div id="refs" class="references">.*', '', x)
+  x = gsub('<div id="refs" class="references[^"]*">.*', '', x)
   x = strip_html(x)
   x = gsub('[[:space:]]', ' ', x)
   x
@@ -280,10 +278,6 @@ strip_search_text = function(x) {
 # manipulate internal options
 opts = knitr:::new_defaults(list(config = list()))
 
-dir_create = function(path) {
-  dir_exists(path) || dir.create(path, recursive = TRUE)
-}
-
 # a wrapper of file.path to ignore `output_dir` if it is NULL
 output_path = function(...) {
   dir = opts$get('output_dir')
@@ -328,7 +322,7 @@ ref_keys_path = function(d = opts$get('output_dir')) {
 #' compiled from a fresh R session, because the state of the current R session
 #' may not be clean.
 #'
-#' For \code{in_sesion = FALSE}, you do not have access to objects in the book
+#' For \code{in_session = FALSE}, you do not have access to objects in the book
 #' from the current R session, but the output is more likely to be reproducible
 #' since everything is created from new R sessions. Since this function is only
 #' for previewing purposes, the cleanness of the R session may not be a big
@@ -386,17 +380,55 @@ serve_book = function(
       if (Rscript(args) != 0) stop('Failed to compile ', paste(files, collapse = ' '))
     }
   }
-  rebuild('index.Rmd', preview_ = FALSE)  # build the whole book initially
+  index <- get_index_file()
+  if (is_empty(index)) {
+    stop("`serve_book()` expects `index.Rmd` in the book project.", call. = FALSE)
+  }
+  rebuild(index, preview_ = FALSE)  # build the whole book initially
   servr::httw('.', ..., site.dir = output_dir, handler = rebuild)
 }
 
+get_index_file <- function() {
+  index_files <- list.files('.', '^index[.]Rmd$', ignore.case = TRUE)
+  if (length(index_files) == 0) return(character())
+  index <- index_files[1]
+  if (length(index_files) > 1) {
+    warning(
+      sprintf(
+        "Several index files found - only one expected. %s will be use, please check your project.",
+        sQuote(index)
+      ))
+  }
+  index
+}
+
 # can only preview HTML output via servr, so look for the first HTML format
 first_html_format = function() {
   fallback = 'bookdown::gitbook'
-  if (!file.exists('index.Rmd')) return(fallback)
-  formats = rmarkdown::all_output_formats('index.Rmd')
-  formats = grep('gitbook|html', formats, value = TRUE)
-  if (length(formats) == 0) fallback else formats[1]
+  html_format = function(f) grep('gitbook|html|bs4_book', f, value = TRUE)
+  get_output_formats(fallback, html_format, first = TRUE)
+}
+
+get_output_formats = function(fallback_format, filter = identity, first = FALSE, fallback_index = NULL) {
+  # Use index files if one exists
+  index = get_index_file()
+  # Use fallback file unless no YAML
+  if (is_empty(index)) {
+    if (length(fallback_index) == 1 &&
+        xfun::file_exists(fallback_index) &&
+        length(rmarkdown::yaml_front_matter(fallback_index)) != 0
+    ) {
+      index = fallback_index
+    } else {
+      return(fallback_format)
+    }
+  }
+  # Retrieve output formats
+  formats = rmarkdown::all_output_formats(index)
+  formats = filter(formats)
+  if (length(formats) == 0) return(fallback_format)
+  if (first) return(formats[1])
+  formats
 }
 
 # base64 encode resources in url("")
@@ -408,7 +440,7 @@ base64_css = function(css, exts = 'png', overwrite = FALSE) {
     if (length(ps) == 0) return(ps)
     ps = gsub('^url\\("|"\\)$', '', ps)
     sprintf('url("%s")', sapply(ps, function(p) {
-      if (grepl(r, p) && file.exists(p)) knitr::image_uri(p) else p
+      if (grepl(r, p) && file.exists(p)) xfun::base64_uri(p) else p
     }))
   })
   if (overwrite) write_utf8(x, css) else x
@@ -416,8 +448,11 @@ base64_css = function(css, exts = 'png', overwrite = FALSE) {
 
 files_cache_dirs = function(dir = '.') {
   if (!dir_exists(dir)) return(character())
-  out = list.files(dir, '_(files|cache)$', full.names = TRUE)
+  r = '_(files|cache)$'
+  out = list.files(dir, r, full.names = TRUE)
   out = out[dir_exists(out)]
+  # only use dirs that have corresponding Rmd files
+  if (dir == '.') out = out[file.exists(sub(r, '.Rmd', out))]
   out = out[basename(out) != '_bookdown_files']
   out
 }
@@ -426,35 +461,38 @@ files_cache_dirs = function(dir = '.') {
 # everything from `from` to `to`, and delete `from`
 move_dir = function(from, to) {
   if (!dir_exists(to)) return(file.rename(from, to))
-  if (file.copy(list.files(from, full.names = TRUE), to, recursive = TRUE))
-    unlink(from, recursive = TRUE)
+  to_copy = list.files(from, full.names = TRUE)
+  if (length(to_copy) == 0 ||
+      any(file.copy(list.files(from, full.names = TRUE), to, recursive = TRUE))
+  ) unlink(from, recursive = TRUE)
+  invisible(TRUE)
 }
 
 move_dirs = function(from, to) mapply(move_dir, from, to)
 
-existing_files = function(x, first = FALSE) {
-  x = x[file.exists(x)]
-  if (first) head(x, 1) else x
-}
-
-existing_r = function(base, first = FALSE) {
+#' @importFrom xfun existing_files
+existing_r = function(base) {
   x = apply(expand.grid(base, c('R', 'r')), 1, paste, collapse = '.')
-  existing_files(x, first)
+  existing_files(x)
 }
 
 target_format = function(format) {
-  if (grepl('(html|gitbook)', format)) return('html')
+  if (grepl('(html|gitbook|bs4)', format)) return('html')
   if (grepl('pdf', format)) return('latex')
+  if (grepl('beamer_', format)) return('latex')
   if (grepl('epub_', format)) return('epub')
   if (grepl('word_', format)) return('docx')
-  switch(format, tufte_book2 = 'latex', tufte_handout2 = 'latex')
+  if (grepl('powerpoint_', format)) return('pptx')
+  switch(format,
+         tufte_book2 = 'latex', `bookdown::tufte_book2` = 'latex',
+         tufte_handout2 = 'latex', `bookdown::tufte_handout2` = "latex")
 }
 
 verify_rstudio_version = function() {
   if (requireNamespace('rstudioapi', quietly = TRUE) && rstudioapi::isAvailable()) {
     if (!rstudioapi::isAvailable('0.99.1200')) warning(
       'Please install a newer version of the RStudio IDE: ',
-      'https://www.rstudio.com/products/rstudio/download/'
+      'https://posit.co/download/rstudio-desktop/'
     )
   } else if (!rmarkdown::pandoc_available('1.17.2')) warning(
     "Please install or upgrade Pandoc to at least version 1.17.2; ",
@@ -469,57 +507,70 @@ str_trim = function(x) gsub('^\\s+|\\s+$', '', x)
 output_md = function() getOption('bookdown.output.markdown', FALSE)
 
 # a theorem engine for knitr (can also be used for lemmas, definitions, etc)
-eng_theorem = function(options) {
-  type = options$type %n% 'theorem'
-  if (!(type %in% names(theorem_abbr))) stop(
-    "The type of theorem '", type, "' is not supported yet."
-  )
-  options$type = type
-  label = paste(theorem_abbr[type], options$label, sep = ':')
-  html.before2 = sprintf('(\\#%s) ', label)
-  name = options$name; to_md = output_md()
-  if (length(name) == 1) {
-    if (to_md) {
-      html.before2 = paste(html.before2, sprintf('(%s) ', name))
-    } else {
-      options$latex.options = sprintf('[%s]', name)
-      html.before2 = paste(html.before2, sprintf('\\iffalse (%s) \\fi{} ', name))
-    }
+eng_theorem = function(type, env) {
+  function(options) {
+    v = if (knitr::pandoc_to(c('epub', 'epub2', 'epub3', 'docx', 'pptx', 'odt'))) '1' else '2'
+    i = sprintf('eng_%s%s', env, v)
+    f = eng_funcs[[i]]
+    f(type, options)
   }
-  options$html.before2 = sprintf(
-    '<span class="%s" id="%s"><strong>%s</strong></span>', type, label, html.before2
-  )
-  process_block(options, to_md)
 }
-
-# a proof engine for unnumbered math environments
-eng_proof = function(options) {
-  type = options$type %n% 'proof'
-  if (!(type %in% names(label_names_math2))) stop(
-    "The type of proof '", type, "' is not supported yet."
-  )
-  options$type = type
-  label = label_prefix(type, label_names_math2)
-  name = options$name; to_md = output_md()
-  if (length(name) == 1) {
-    if (!to_md) options$latex.options = sprintf('[%s]', sub('[.]\\s*$', '', name))
-    r = '^(.+?)([[:punct:][:space:]]+)$'  # "Remark. " -> "Remark (Name). "
-    if (grepl(r, label)) {
-      label1 = gsub(r, '\\1', label)
-      label2 = paste0(' (', name, ')', gsub(r, '\\2', label))
-    } else {
-      label1 = label; label2 = ''
+# TODO: remove eng_theorem1(), eng_proof1(), and process_block() when
+# https://github.com/rstudio/bookdown/issues/1179 is resolved
+eng_funcs = list(
+  eng_theorem1 = function(type, options) {
+    options$type = type
+    label = paste(theorem_abbr[type], options$label, sep = ':')
+    html.before2 = sprintf('(\\#%s) ', label)
+    name = options$name; to_md = output_md()
+    if (length(name) == 1) {
+      if (to_md) {
+        html.before2 = paste(html.before2, sprintf('(%s) ', name))
+      } else {
+        options$latex.options = sprintf('[%s]', name)
+        html.before2 = paste(html.before2, sprintf('\\iffalse (%s) \\fi{} ', name))
+      }
     }
-    label = sprintf('<em>%s</em>%s', label1, label2)
-  } else {
-    label = sprintf('<em>%s</em>', label)
+    options$html.before2 = sprintf(
+      '<span class="%s" id="%s"><strong>%s</strong></span>', type, label, html.before2
+    )
+    process_block(options, to_md)
+  },
+  eng_theorem2 = function(type, options) {
+    label = paste0('#', options$label)
+    name = sprintf('name="%s"', options$name)
+    res = paste(c(paste0('.', type), label, name), collapse = ' ')
+    paste(c(sprintf('::: {%s}', res), options$code, ':::'), collapse = '\n')
+  },
+  eng_proof1 = function(type, options) {
+    options$type = type
+    label = label_prefix(type, label_names_math2)()
+    name = options$name; to_md = output_md()
+    if (length(name) == 1) {
+      if (!to_md) options$latex.options = sprintf('[%s]', sub('[.]\\s*$', '', name))
+      r = '^(.+?)([[:punct:][:space:]]+)$'  # "Remark. " -> "Remark (Name). "
+      if (grepl(r, label)) {
+        label1 = gsub(r, '\\1', label)
+        label2 = paste0(' (', name, ')', gsub(r, '\\2', label))
+      } else {
+        label1 = label; label2 = ''
+      }
+      label = sprintf('<em>%s</em>%s', label1, label2)
+    } else {
+      label = sprintf('<em>%s</em>', label)
+    }
+    options$html.before2 = sprintf(
+      '<span class="%s">%s</span> ', type, label
+    )
+    if (!to_md) options$html.before2 = paste('\\iffalse{}', options$html.before2, '\\fi{}')
+    process_block(options, to_md)
+  },
+  eng_proof2 = function(type, options) {
+    name = sprintf('name="%s"', options$name)
+    res = paste(c(paste0('.', type), name), collapse = ' ')
+    paste(c(sprintf('::: {%s}', res), options$code, ':::'), collapse = '\n')
   }
-  options$html.before2 = sprintf(
-    '<span class="%s">%s</span> ', type, label
-  )
-  if (!to_md) options$html.before2 = paste('\\iffalse{}', options$html.before2, '\\fi{}')
-  process_block(options, to_md)
-}
+)
 
 process_block = function(options, md) {
   if (md) {
@@ -532,13 +583,11 @@ process_block = function(options, md) {
   knitr:::eng_block2(options)
 }
 
-register_eng_math = function(envs, engine) {
-  knitr::knit_engines$set(setNames(lapply(envs, function(env) {
-    function(options) {
-      options$type = env
-      engine(options)
-    }
-  }), envs))
+register_eng_math = function() {
+  lapply(c('theorem', 'proof'), function(env) {
+    envs = names(if (env == 'theorem') theorem_abbr else label_names_math2)
+    knitr::knit_engines$set(setNames(lapply(envs, eng_theorem, env = env), envs))
+  })
 }
 
 pandoc2.0 = function() rmarkdown::pandoc_available('2.0')
@@ -574,8 +623,8 @@ lua_filter = function (filters = NULL) {
 
 # pass _bookdown.yml to Pandoc's Lua filters
 bookdown_yml_arg = function(config = load_config(), path = tempfile()) {
-  # this is supported for Pandoc >= 2.0 only
-  if (!pandoc2.0() || length(config) == 0) return()
+  # this is supported for Pandoc >= 2.3 only
+  if (!rmarkdown::pandoc_available('2.3') || length(config) == 0) return()
   yaml::write_yaml(list(bookdown = config), path)
   c("--metadata-file", rmarkdown::pandoc_path_arg(path))
 }
@@ -619,15 +668,26 @@ fence_theorems = function(input, text = xfun::read_utf8(input), output = NULL) {
   block_end = vapply(block_start, function(x) block_end[block_end > x][1], integer(1))
   # add a . to engine name
   params = sprintf(".%s", params)
-  # change label to id
-  params = gsub(",\\s*([-/[:alnum:]]+)(,|\\s*$)", " #\\1", params)
+  # change implicit label to id
+  params = gsub("^([.][a-zA-Z0-9_]+(?:\\s*,\\s*|\\s+))([-/[:alnum:]]+)(\\s*,|\\s*$)", "\\1#\\2", params)
+  # change explicit label to id
   params = gsub("label\\s*=\\s*\"([-/[:alnum:]]+)\"", "#\\1", params)
-  # clean , and space
-  params = gsub(",\\s*", " ", params)
+  # clean , and spaces
+  params = gsub("\\s*,\\s*", " ", params)
   params = gsub("\\s*=\\s*", "=", params)
   # modify the blocks
   text[block_start] = sprintf("::: {%s}", params)
   text[block_end] = ":::"
   # return the text or write to output file
-  if (is.null(output)) xfun::raw_string(text) else xfun::write_utf8(text, input)
+  if (is.null(output)) xfun::raw_string(text) else xfun::write_utf8(text, output)
+}
+
+stop_if_not_exists = function(inputs) {
+  if (!all(exist <- xfun::file_exists(inputs))) {
+    stop("Some files were not found: ",  paste(inputs[!exist], collapse = ' '))
+  }
+}
+
+is_empty = function(x) {
+  length(x) == 0 || !nzchar(x)
 }
